package fernandoalvarez.eventizer.Chats


import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import fernandoalvarez.eventizer.Model.Event
import fernandoalvarez.eventizer.Model.Message
import fernandoalvarez.eventizer.R
import kotlinx.android.synthetic.main.fragment_chat.*

class ChatFragment : Fragment() {

    private var chatPresenter: ChatPresenter? = null
    private var layoutManager: RecyclerView.LayoutManager? = null
    private var adapter: RecyclerView.Adapter<MessagesAdapter.ViewHolder>? = null
    private var messages = ArrayList<Message>()
    private lateinit var event: Event
    private lateinit var userId: String
    private lateinit var userName: String
    private var isAdmin: Boolean = false

    fun newInstance(event: Event, userId:String): ChatFragment {
        val chatFragment = ChatFragment()
        val args = Bundle()
        args.putSerializable("event",event)
        args.putString("userId",userId)
        chatFragment.arguments = args
        return chatFragment
    }

    override fun onStart() {
        super.onStart()
        listenersSetup()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        event = arguments!!.get("event") as Event
        userId = arguments!!.getString("userId")
        if (chatPresenter == null)
        {
            chatPresenter = ChatPresenter(this)
        }
        chatPresenter!!.eventId = event.id
        chatPresenter!!.getUsername()
        checkAdmin()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        val view = inflater.inflate(R.layout.fragment_chat, container, false)

        layoutManager = LinearLayoutManager(context)

        val rv = view.findViewById(R.id.messagesList) as RecyclerView
        rv.layoutManager = layoutManager

        adapter = MessagesAdapter(messages, this::messageClicked, isAdmin)
        rv.adapter = adapter

        messages.clear()
        listenForChanges()

        return view
    }

    private fun messageClicked(message: Message)
    {
        chatPresenter?.deleteMessage(message,event.id)
        //Toast.makeText(this.context, message.id + " contact clicked", Toast.LENGTH_SHORT).show()
    }

    fun addMessage(message: Message)
    {
        messages.add(message)
        this.adapter?.notifyDataSetChanged()
    }

    fun deleteMessage(message: Message)
    {
        messages.remove(message)
        this.adapter?.notifyDataSetChanged()
    }

    private fun checkAdmin() {
        if (userId == event.admin)
            isAdmin=true
    }

    fun setUsernameInfo(userName: String)
    {
        this.userName = userName
    }

    private fun listenersSetup()
    {
        messageSendButton.setOnClickListener { sendMessage() }
    }

    private fun listenForChanges()
    {
        chatPresenter?.listenForChanges()
    }

   private fun sendMessage()
    {
        val text: String = messageTextInput.text.toString()
        if (text != "")
        {
            val message = Message()
            message.text = text
            message.timestamp = System.currentTimeMillis().toString()
            message.sender = userName
            chatPresenter?.sendMessage(message)
            messageTextInput.text.clear()
        }

    }

}
