package fernandoalvarez.eventizer.Events

import android.app.Activity
import android.app.DatePickerDialog
import android.app.TimePickerDialog
import android.content.Context
import android.content.Intent
import android.location.LocationManager
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import com.google.android.gms.common.api.GoogleApiClient
import android.provider.Settings
import android.support.v7.app.AlertDialog
import android.view.View
import android.widget.Toast
import com.google.android.gms.location.places.Place
import java.text.SimpleDateFormat
import kotlinx.android.synthetic.main.activity_add_event.*
import java.util.*
import com.google.android.gms.location.places.Places
import com.google.android.gms.location.places.ui.PlacePicker
import fernandoalvarez.eventizer.Model.Event
import fernandoalvarez.eventizer.R

class AddEventActivity : AppCompatActivity() {

    private lateinit var addEventPresenter: AddEventPresenter

    private val PLACE_PICKER_REQUEST = 1
    private var mGoogleApiClient: GoogleApiClient? = null
    private lateinit var selectedPlace: Place
    private var datePickerCal: Calendar = Calendar.getInstance()
    private var timePickerCal: Calendar = Calendar.getInstance()


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_add_event)
        addEventPresenter = AddEventPresenter(this)
        buildGoogleApiClient()
        listenersSetup()
    }

    private fun listenersSetup()
    {
        val datePickerListener = DatePickerDialog.OnDateSetListener { _, year, monthOfYear, dayOfMonth ->
            datePickerCal.set(Calendar.YEAR, year)
            datePickerCal.set(Calendar.MONTH, monthOfYear)
            datePickerCal.set(Calendar.DAY_OF_MONTH, dayOfMonth)
            updateDateInView()
        }

        eventDateButton.setOnClickListener {
            DatePickerDialog(this@AddEventActivity,
                    datePickerListener,
                    datePickerCal.get(Calendar.YEAR),
                    datePickerCal.get(Calendar.MONTH),
                    datePickerCal.get(Calendar.DAY_OF_MONTH)).show()
        }

        eventTimeButton.setOnClickListener {
            val timePickerListener = TimePickerDialog.OnTimeSetListener{ _, hour, minute ->
                timePickerCal.set(Calendar.HOUR_OF_DAY,hour)
                timePickerCal.set(Calendar.MINUTE,minute)
                eventTimeValue.text = SimpleDateFormat("HH:mm", Locale.FRANCE).format(timePickerCal.time)
            }
            TimePickerDialog(this, timePickerListener,timePickerCal.get(Calendar.HOUR_OF_DAY),timePickerCal.get(Calendar.MINUTE),true).show()
        }

        eventLocationButton.setOnClickListener(View.OnClickListener {
            if (!checkGPSEnabled()) {
                return@OnClickListener
            }
            val builder = PlacePicker.IntentBuilder()
            startActivityForResult(builder.build(this), PLACE_PICKER_REQUEST)
        })

        submit.setOnClickListener{ submit() }
    }

    private fun updateDateInView() {
        val myFormat = "dd/MM/yyyy"
        val sdf = SimpleDateFormat(myFormat, Locale.FRANCE)
        eventDateValue.text = sdf.format(datePickerCal.time).toString()
    }

    private fun checkGPSEnabled(): Boolean {
        if (!isLocationEnabled())
            showAlert()
        return isLocationEnabled()
    }

    private fun isLocationEnabled(): Boolean {
        val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
        return locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER) || locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
    }

    private fun showAlert() {
        val dialog = AlertDialog.Builder(this)
        dialog.setTitle("Enable Location")
                .setMessage("Locations Settings is set to 'Off'.\nEnable Location to use this app")
                .setPositiveButton("Location Settings") { _, _ ->
                    val myIntent = Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)
                    startActivity(myIntent)
                }
                .setNegativeButton("Cancel") { _, _ -> }
        dialog.show()
    }

    private fun buildGoogleApiClient() {
        mGoogleApiClient = GoogleApiClient.Builder(this)
                .addApi(Places.GEO_DATA_API)
                .addApi(Places.PLACE_DETECTION_API)
                .enableAutoManage(this, null)
                .build()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == PLACE_PICKER_REQUEST && resultCode == Activity.RESULT_OK) {
            selectedPlace = PlacePicker.getPlace(this,data)
            eventPlaceValueName.text = selectedPlace.name.toString()
            eventPlaceValueAdress.text = selectedPlace.address.toString()
        }
    }

    private fun submit()
    {
        val event = Event()
        event.title = eventTitleField.text.toString()
        event.description = eventDescriptionField.text.toString()
        event.date = eventDateValue.text.toString()
        event.time = eventTimeValue.text.toString()
        if (event.title != "" && event.description != "" && selectedPlace.name != null)
        {
            event.latitude = selectedPlace.latLng.latitude.toString()
            event.longitude = selectedPlace.latLng.longitude.toString()
            addEventPresenter.eventSubmitted(event)
            this.finish()
        }
        else
        {
            Toast.makeText(this,"Todos los campos son obligatorios",Toast.LENGTH_SHORT).show()
        }
    }

}
