package fernandoalvarez.eventizer.Events

import android.content.Intent
import android.os.Bundle
import android.provider.CalendarContract
import android.support.v4.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.google.android.gms.maps.CameraUpdateFactory
import com.google.android.gms.maps.model.LatLng
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.OnMapReadyCallback
import com.google.android.gms.maps.model.MarkerOptions
import fernandoalvarez.eventizer.R
import com.google.android.gms.maps.MapView
import fernandoalvarez.eventizer.Contacts.ContactsSelectionActivity
import fernandoalvarez.eventizer.Model.Event
import kotlinx.android.synthetic.main.fragment_event_detail.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.math.absoluteValue


class EventDetailFragment : Fragment(), OnMapReadyCallback {

    private var mMap: MapView? = null
    private lateinit var event:Event
    private lateinit var userId: String

    fun newInstance(event: Event, userId:String): EventDetailFragment {
        val eventDetailFragment = EventDetailFragment()
        val args = Bundle()
        args.putSerializable("event",event)
        args.putString("userId",userId)
        eventDetailFragment.arguments = args
        return eventDetailFragment
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        mMap?.onSaveInstanceState(outState)
    }

    override fun onStart() {
        super.onStart()
        mMap?.onStart()
        setEventData()
        checkAdmin()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        event = arguments!!.get("event") as Event
        userId = arguments!!.getString("userId")
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {

        val view = inflater.inflate(R.layout.fragment_event_detail, container, false)

        mMap = view?.findViewById(R.id.mapview) as MapView
        mMap?.onCreate(savedInstanceState)
        return view
    }

   private fun setEventData()
    {
        eventTitleValue.text = event.title
        eventDescriptionValue.text = event.description
        eventDateValue.text = event.date
        eventTimeValue.text = event.time
        mMap?.getMapAsync(this)
        listenersSetup()
    }

    private fun checkAdmin()
    {
        if (userId == event.admin)
            showAdminUIElements()
    }

    private fun showAdminUIElements()
    {
        manageEventUsersText.visibility = View.VISIBLE
        manageEventUsersButton.visibility = View.VISIBLE
    }

    override fun onMapReady(googleMap: GoogleMap) {
        with(googleMap) {
            moveCamera(CameraUpdateFactory.newLatLngZoom(LatLng(event.latitude.toDouble(), event.longitude.toDouble()), 15f))
            addMarker(MarkerOptions().position(LatLng(event.latitude.toDouble(), event.longitude.toDouble())))
        }
    }

    private fun listenersSetup()
    {
        eventExportButton.setOnClickListener { exportEventToCalendar() }
        manageEventUsersButton.setOnClickListener { val intent = Intent(activity, ContactsSelectionActivity::class.java)
            intent.putExtra("eventId", event.id)
            startActivity(intent)
        }
    }

    private fun exportEventToCalendar()
    {
        val intent = Intent(Intent.ACTION_EDIT)
        intent.data = CalendarContract.Events.CONTENT_URI
        intent.putExtra(CalendarContract.Events.TITLE, event.title)
        intent.putExtra(CalendarContract.Events.DESCRIPTION, event.description)
        intent.putExtra(CalendarContract.EXTRA_EVENT_BEGIN_TIME, dateToUTC(event))
        intent.putExtra(CalendarContract.Events.EVENT_TIMEZONE, TimeZone.getDefault())
        intent.putExtra(CalendarContract.Events.CALENDAR_TIME_ZONE, TimeZone.getDefault())
        intent.putExtra(CalendarContract.Events.EVENT_LOCATION, event.latitude+", "+event.longitude)
        startActivity(intent)
    }

    private fun dateToUTC(event: Event): Long {
        val dateStr = "${event.date} ${event.time}"
        val inputFormat = SimpleDateFormat("dd/MM/yyyy HH:mm", Locale.getDefault())
        inputFormat.timeZone = TimeZone.getDefault()
        val inputDate = inputFormat.parse(dateStr)
        return inputDate.time
    }

    override fun onResume() {
        super.onResume()
        mMap?.onResume()
    }

    override fun onPause() {
        super.onPause()
        mMap?.onPause()
    }

    override fun onStop() {
        super.onStop()
        mMap?.onStop()
    }

    override fun onDestroy() {
        super.onDestroy()
        mMap?.onDestroy()
    }

    override fun onLowMemory() {
        super.onLowMemory()
        mMap?.onLowMemory()
    }

}
