package fernandoalvarez.eventizer

import android.util.Log
import com.google.android.gms.tasks.OnSuccessListener
import com.google.android.gms.tasks.Task
import com.google.firebase.auth.AuthResult
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.*
import fernandoalvarez.eventizer.Model.Event
import fernandoalvarez.eventizer.Model.Message
import fernandoalvarez.eventizer.Model.User


object FirebaseUtil {

    private var usersReference: DatabaseReference? = null
    private var contactsReference: DatabaseReference? = null
    private var eventsReference: DatabaseReference? = null
    private var eventsInvitesReference: DatabaseReference? = null
    private var messagesReference: DatabaseReference? = null
    private var mDatabase: FirebaseDatabase? = null
    private var mAuth: FirebaseAuth? = null

    init {
        mDatabase = FirebaseDatabase.getInstance()
        usersReference = mDatabase?.reference?.child("Users")
        contactsReference = mDatabase!!.reference!!.child("Contacts")
        eventsReference = mDatabase!!.reference!!.child("Events")
        eventsInvitesReference = mDatabase!!.reference!!.child("EventsInvites")
        messagesReference = mDatabase!!.reference!!.child("Messages")
        mAuth = FirebaseAuth.getInstance()
    }

    fun registerUserAuth(email: String, password: String): Task<AuthResult> {
        return mAuth!!.createUserWithEmailAndPassword(email, password)
    }

    fun registerUserDb(email: String, username: String) {
        val currentUser = usersReference!!.child(mAuth!!.currentUser!!.uid)
        currentUser.child("email").setValue(email)
        currentUser.child("username").setValue(username)
    }

    fun performLogin(email: String, password: String): Task<AuthResult> {
        return mAuth!!.signInWithEmailAndPassword(email, password)
                .addOnCompleteListener { task: Task<AuthResult> ->
                    if (task.isSuccessful) {
                        Log.d("finished", "performLogin")
                    }
                }
    }

    fun performLogout() {
        mAuth?.signOut()
    }

    fun checkAuth(): Boolean {
        val user = mAuth!!.currentUser
        return user != null
    }

    fun getUserByMail(listener: ValueEventListener, email: String) {
        val q = usersReference!!.ref!!.orderByChild("email")!!.equalTo(email)
        q.addListenerForSingleValueEvent(listener)
    }

    fun getUserById(listener: ValueEventListener, id: String) {
        val q = usersReference!!.child(id)
        q.addListenerForSingleValueEvent(listener)
    }

    fun addNewContact(user: User)
    {
        val currentFriend = contactsReference!!.child(mAuth!!.currentUser!!.uid)
        currentFriend.child(user.id).setValue(true)
    }

    fun deleteContact(user: User, listener: OnSuccessListener<Void>)
    {
        val q=contactsReference!!.child(mAuth!!.currentUser!!.uid).child(user.id).removeValue()
        q.addOnSuccessListener(listener)
    }

    fun getContactsIds(listener: ValueEventListener)
    {
        val q = contactsReference!!.child(mAuth!!.currentUser!!.uid)
        q.addListenerForSingleValueEvent(listener)
    }

    fun addNewEvent(event: Event, listener: OnSuccessListener<Void>)
    {
        val key = eventsReference!!.push().key
        event.id=key
        event.admin= mAuth!!.currentUser!!.uid
        eventsReference!!.child(key).setValue(event)
        eventsReference!!.child(key).child("membersList").child(mAuth!!.currentUser!!.uid).setValue(true)

        val q = usersReference!!.child(mAuth!!.currentUser!!.uid).child("eventsList").child(key).setValue(true)
        q.addOnSuccessListener(listener)
    }

    fun getEventById(listener: ValueEventListener,id:String)
    {
        val q = eventsReference!!.child(id)
        q.addListenerForSingleValueEvent(listener)
    }


    fun getEventsInvites(listener: ValueEventListener)
    {
        val q = eventsInvitesReference!!.child(mAuth!!.currentUser!!.uid)
        q.addListenerForSingleValueEvent(listener)
    }


    fun getEventsIds(listener: ValueEventListener) {
        val q = usersReference!!.child(mAuth!!.currentUser!!.uid).child("eventsList")
        q.addListenerForSingleValueEvent(listener)
    }

    fun addNewMessage(listener: OnSuccessListener<Void>, eventId: String, message: Message)
    {
        val key = messagesReference!!.child(eventId).push().key
        message.id = key
        val q = messagesReference!!.child(eventId).child(key).setValue(message)
        q.addOnSuccessListener(listener)

    }

    fun getMessages(listener: ValueEventListener, eventId:String)
    {
        val q = messagesReference!!.child(eventId)
        q.addListenerForSingleValueEvent(listener)
    }

    fun deleteMessage(listener: OnSuccessListener<Void>,eventId:String, messageId:String)
    {
        val q = messagesReference!!.child(eventId).child(messageId).removeValue()
        q.addOnSuccessListener(listener)
    }

    fun getUsername(listener: ValueEventListener)
    {
        val q = usersReference!!.child(mAuth!!.currentUser!!.uid).child("username")
        q.addListenerForSingleValueEvent(listener)
    }

    fun getUsersFromEvent(listener: ValueEventListener, eventId:String)
    {
       val q = eventsReference!!.child(eventId).child("membersList")
        q.addListenerForSingleValueEvent(listener)
    }

    fun removeUserFromEvent(listener: OnSuccessListener<Void>, eventId:String, userId:String)
    {
        usersReference!!.child(userId).child("eventsList").child(eventId).removeValue()
        val q=eventsReference!!.child(eventId).child("membersList").child(userId).removeValue()
        q.addOnSuccessListener(listener)
    }

    fun inviteUserToEvent(listener: OnSuccessListener<Void>, eventId:String, userId:String)
    {
       val q = eventsInvitesReference!!.child(userId).child(eventId).setValue(true)
        q.addOnSuccessListener(listener)
    }

    fun acceptEventInvite(listener: OnSuccessListener<Void>, eventId:String)
    {
        usersReference!!.child(mAuth!!.currentUser!!.uid).child("eventsList").child(eventId).setValue(true)
        eventsReference!!.child(eventId).child("membersList").child(mAuth!!.currentUser!!.uid).setValue(true)
        val q = eventsInvitesReference!!.child(mAuth!!.currentUser!!.uid).child(eventId).removeValue()
        q.addOnSuccessListener(listener)
    }

    fun refuseEventInvite(listener: OnSuccessListener<Void>, eventId:String)
    {
        val q = eventsInvitesReference!!.child(mAuth!!.currentUser!!.uid).child(eventId).removeValue()
        q.addOnSuccessListener(listener)
    }

    fun listenForEventsChanges(listener: ChildEventListener)
    {
        val q = usersReference!!.child(mAuth!!.currentUser!!.uid).child("eventsList")
        q.addChildEventListener(listener)
    }

    fun listenForMessagesChanges(listener: ChildEventListener, eventId:String)
    {
        val q = messagesReference!!.child(eventId)
        q.addChildEventListener(listener)
    }

    fun listenForInvitesChanges(listener: ChildEventListener)
    {
        val q = eventsInvitesReference!!.child(mAuth!!.currentUser!!.uid)
        q.removeEventListener(listener)
        q.addChildEventListener(listener)
    }

    fun getUserId(): String {
        return mAuth!!.currentUser!!.uid
    }

}

