package fernandoalvarez.eventizer.Login

import android.content.Intent
import android.support.v7.app.AppCompatActivity
import android.os.Bundle
import android.util.Patterns
import android.widget.Toast
import fernandoalvarez.eventizer.FirebaseUtil
import fernandoalvarez.eventizer.MainMenu.MainMenuActivity
import fernandoalvarez.eventizer.R
import fernandoalvarez.eventizer.Register.RegisterActivity
import kotlinx.android.synthetic.main.activity_login.*
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.RegexTemplate

class LoginActivity : AppCompatActivity() {

    private lateinit var loginPresenter : LoginPresenter
    private var awesomeValidation: AwesomeValidation? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC)
        setContentView(R.layout.activity_login)
        addValidation()
        loginPresenter = LoginPresenter(this)
        if (!FirebaseUtil.checkAuth())
        {
            listenersSetup()
        }
        else
        {
            goToMainMenu()
        }

    }

    private fun listenersSetup()
    {
        loginButton.setOnClickListener {
            if (awesomeValidation!!.validate())
            loginPresenter.performLogin(emailField.text.toString(), passwordField.text.toString())
        }
        registerButton.setOnClickListener{ goToRegisterForm() }
    }

    fun showBadCredentialsMessage()
    {
        Toast.makeText(this,getString(R.string.credentials_validation_error), Toast.LENGTH_SHORT).show()
    }

    fun goToMainMenu() {
        val intent = Intent(this, MainMenuActivity::class.java)
        startActivity(intent)
    }

    private fun goToRegisterForm() {
        val intent = Intent(this, RegisterActivity::class.java)
        startActivity(intent)
    }

    private fun addValidation()
    {
        awesomeValidation?.addValidation(emailField, Patterns.EMAIL_ADDRESS, getString(R.string.mail_validation_error))
        awesomeValidation?.addValidation(passwordField, RegexTemplate.NOT_EMPTY, getString(R.string.password_validation_error))
    }

}
